@@ -14,14 +14,20 @@
 std::string metrics_path = "";
 struct amdgpu_common_metrics amdgpu_common_metrics;
 std::mutex amdgpu_common_metrics_m;
+std::mutex amdgpu_m;
+std::condition_variable amdgpu_c;
+bool amdgpu_run_thread = true;
+std::unique_ptr<Throttling> throttling;
 
 bool amdgpu_verify_metrics(const std::string& path)
 {
 	metrics_table_header header {};
 	FILE *f;
 	f = fopen(path.c_str(), "rb");
-	if (!f)
+	if (!f) {
+		SPDLOG_DEBUG("Failed to read the metrics header of '{}'", path);
 		return false;
+	}
 
 	if (fread(&header, sizeof(header), 1, f) == 0)
 	{
@@ -36,8 +42,8 @@ bool amdgpu_verify_metrics(const std::string& path)
 				break;
 			cpuStats.cpu_type = "GPU";
 			return true;
-		case 2: // v2_1, v2_2, v2_3
-			if(header.content_revision<=0 || header.content_revision>3)// v2_0, not naturally aligned
+		case 2: // v2_1, v2_2, v2_3, v2_4
+			if(header.content_revision<=0 || header.content_revision>4)// v2_0, not naturally aligned
 				break;
 			cpuStats.cpu_type = "APU";
 			return true;
@@ -49,11 +55,10 @@ bool amdgpu_verify_metrics(const std::string& path)
 	return false;
 }
 
-#define MAX(x, y) (((x) > (y)) ? (x) : (y))
 #define IS_VALID_METRIC(FIELD) (FIELD != 0xffff)
 void amdgpu_get_instant_metrics(struct amdgpu_common_metrics *metrics) {
 	FILE *f;
-	void *buf[MAX(sizeof(struct gpu_metrics_v1_3), sizeof(struct gpu_metrics_v2_3))/sizeof(void*)+1];
+	void *buf[MAX(sizeof(struct gpu_metrics_v1_3), sizeof(struct gpu_metrics_v2_4))/sizeof(void*)+1];
 	struct metrics_table_header* header = (metrics_table_header*)buf;
 
 	f = fopen(metrics_path.c_str(), "rb");
@@ -81,6 +86,7 @@ void amdgpu_get_instant_metrics(struct amdgpu_common_metrics *metrics) {
 
 		metrics->gpu_temp_c = amdgpu_metrics->temperature_edge;
 		indep_throttle_status = amdgpu_metrics->indep_throttle_status;
+		metrics->fan_speed = amdgpu_metrics->current_fan_speed;
 	} else if (header->format_revision == 2) {
 		// APUs
 		struct gpu_metrics_v2_3 *amdgpu_metrics = (struct gpu_metrics_v2_3 *) buf;
@@ -179,6 +185,8 @@ void amdgpu_get_instant_metrics(struct amdgpu_common_metrics *metrics) {
 	metrics->is_current_throttled = ((indep_throttle_status >> 16) & 0xFF) != 0;
 	metrics->is_temp_throttled = ((indep_throttle_status >> 32) & 0xFFFF) != 0;
 	metrics->is_other_throttled = ((indep_throttle_status >> 56) & 0xFF) != 0;
+	if (throttling)
+		throttling->indep_throttle_status = indep_throttle_status;
 }
 
 void amdgpu_get_samples_and_copy(struct amdgpu_common_metrics metrics_buffer[METRICS_SAMPLE_COUNT], bool &gpu_load_needs_dividing) {
@@ -212,6 +220,8 @@ void amdgpu_get_samples_and_copy(struct amdgpu_common_metrics metrics_buffer[MET
 		UPDATE_METRIC_MAX(is_current_throttled);
 		UPDATE_METRIC_MAX(is_temp_throttled);
 		UPDATE_METRIC_MAX(is_other_throttled);
+
+		UPDATE_METRIC_MAX(fan_speed);
 		amdgpu_common_metrics_m.unlock();
 }
 
@@ -230,6 +240,9 @@ void amdgpu_metrics_polling_thread() {
 	memset(metrics_buffer, 0, sizeof(metrics_buffer));
 
 	while (1) {
+		std::unique_lock<std::mutex> lock(amdgpu_m);
+		amdgpu_c.wait(lock, []{return amdgpu_run_thread;});
+		lock.unlock();
 #ifndef TEST_ONLY
 		if (HUDElements.params->no_display && !logger->is_active())
 			usleep(100000);
@@ -239,7 +252,7 @@ void amdgpu_metrics_polling_thread() {
 	}
 }
 
-void amdgpu_get_metrics(){
+void amdgpu_get_metrics(uint32_t deviceID){
 	static bool init = false;
 	if (!init){
 		std::thread(amdgpu_metrics_polling_thread).detach();
@@ -250,10 +263,15 @@ void amdgpu_get_metrics(){
 	gpu_info.load = amdgpu_common_metrics.gpu_load_percent;
 
 	gpu_info.powerUsage = amdgpu_common_metrics.average_gfx_power_w;
-	gpu_info.CoreClock = amdgpu_common_metrics.current_gfxclk_mhz;
 	gpu_info.MemClock = amdgpu_common_metrics.current_uclk_mhz;
 
 	// Use hwmon instead, see gpu.cpp
+	if ( deviceID == 0x1435 || deviceID == 0x163f )
+	{
+		// If we are on VANGOGH (Steam Deck), then
+		// always use use core clock from GPU metrics.
+		gpu_info.CoreClock = amdgpu_common_metrics.current_gfxclk_mhz;
+	}
 	// gpu_info.temp = amdgpu_common_metrics.gpu_temp_c;
 	gpu_info.apu_cpu_power = amdgpu_common_metrics.average_cpu_power_w;
 	gpu_info.apu_cpu_temp = amdgpu_common_metrics.apu_cpu_temp_c;
@@ -263,5 +281,7 @@ void amdgpu_get_metrics(){
 	gpu_info.is_temp_throttled = amdgpu_common_metrics.is_temp_throttled;
 	gpu_info.is_other_throttled = amdgpu_common_metrics.is_other_throttled;
 
+	gpu_info.fan_speed = amdgpu_common_metrics.fan_speed;
+
 	amdgpu_common_metrics_m.unlock();
 }
