@@ -25,6 +25,7 @@ static  std::vector<std::string> blacklist {
     "EpicGamesLauncher.exe",
     "IGOProxy.exe",
     "IGOProxy64.exe",
+    "monado-service",
     "Origin.exe",
     "OriginThinSetupInternal.exe",
     "steam",
@@ -46,7 +47,16 @@ static  std::vector<std::string> blacklist {
     "REDprelauncher.exe",
     "REDlauncher.exe",
     "gamescope",
-    "RSI Launcher.exe"
+    "RSI Launcher.exe",
+    "tabtip.exe",
+    "steam.exe",
+    "explorer.exe",
+    "wine-preloader",
+    "iexplore.exe",
+    "rundll32.exe",
+    "Launcher", //Paradox Interactive Launcher
+    "steamwebhelper.exe",
+    "EpicWebHelper.exe"
 };
 
 
@@ -55,7 +65,9 @@ static bool check_blacklisted() {
     global_proc_name = proc_name;
     bool blacklisted = std::find(blacklist.begin(), blacklist.end(), proc_name) != blacklist.end();
 
-    if(blacklisted) {
+    static bool printed = false;
+    if(blacklisted && !printed) {
+        printed = true;
         SPDLOG_INFO("process '{}' is blacklisted in MangoHud", proc_name);
     }
 
