@@ -25,6 +25,10 @@
 #define PROCMEMINFOFILE PROCDIR "/meminfo"
 #endif
 
+#ifndef PROCCPUINFOFILE
+#define PROCCPUINFOFILE PROCDIR "/cpuinfo"
+#endif
+
 #include "file_utils.h"
 
 static void calculateCPUData(CPUData& cpuData,
@@ -226,15 +230,33 @@ bool CPUStats::UpdateCPUData()
 bool CPUStats::UpdateCoreMhz() {
     m_coreMhz.clear();
     FILE *fp;
-    for (auto& cpu : m_cpuData)
-    {
-        std::string path = "/sys/devices/system/cpu/cpu" + std::to_string(cpu.cpu_id) + "/cpufreq/scaling_cur_freq";
-        if ((fp = fopen(path.c_str(), "r"))){
-            int64_t temp;
-            if (fscanf(fp, "%" PRId64, &temp) != 1)
-                temp = 0;
-            cpu.mhz = temp / 1000;
-            fclose(fp);
+    static bool scaling_freq = true;
+    if (scaling_freq){
+        for (auto& cpu : m_cpuData){
+            std::string path = "/sys/devices/system/cpu/cpu" + std::to_string(cpu.cpu_id) + "/cpufreq/scaling_cur_freq";
+            if ((fp = fopen(path.c_str(), "r"))){
+                int64_t temp;
+                if (fscanf(fp, "%" PRId64, &temp) != 1)
+                    temp = 0;
+                cpu.mhz = temp / 1000;
+                fclose(fp);
+                scaling_freq = true;
+            } else {
+                scaling_freq = false;
+                break;
+            }
+        }
+    } else {
+        static std::ifstream cpuInfo(PROCCPUINFOFILE);
+        static std::string row;
+        size_t i = 0;
+        while (std::getline(cpuInfo, row) && i < m_cpuData.size()) {
+            if (row.find("MHz") != std::string::npos){
+                row = std::regex_replace(row, std::regex(R"([^0-9.])"), "");
+                if (!try_stoi(m_cpuData[i].mhz, row))
+                    m_cpuData[i].mhz = 0;
+                i++;
+            }
         }
     }
 
@@ -274,9 +296,23 @@ bool CPUStats::UpdateCpuTemp() {
 static bool get_cpu_power_k10temp(CPUPowerData* cpuPowerData, float& power) {
     CPUPowerData_k10temp* powerData_k10temp = (CPUPowerData_k10temp*)cpuPowerData;
 
+    if(powerData_k10temp->corePowerFile || powerData_k10temp->socPowerFile)
+    {
+        rewind(powerData_k10temp->corePowerFile);
+        rewind(powerData_k10temp->socPowerFile);
+        fflush(powerData_k10temp->corePowerFile);
+        fflush(powerData_k10temp->socPowerFile);
+        int corePower, socPower;
+        if (fscanf(powerData_k10temp->corePowerFile, "%d", &corePower) != 1)
+            goto voltagebased;
+        if (fscanf(powerData_k10temp->socPowerFile, "%d", &socPower) != 1)
+            goto voltagebased;
+        power = (corePower + socPower) / 1000000;
+        return true;
+    }
+    voltagebased:
     if (!powerData_k10temp->coreVoltageFile || !powerData_k10temp->coreCurrentFile || !powerData_k10temp->socVoltageFile || !powerData_k10temp->socCurrentFile)
         return false;
-
     rewind(powerData_k10temp->coreVoltageFile);
     rewind(powerData_k10temp->coreCurrentFile);
     rewind(powerData_k10temp->socVoltageFile);
@@ -328,6 +364,33 @@ static bool get_cpu_power_zenpower(CPUPowerData* cpuPowerData, float& power) {
     return true;
 }
 
+static bool get_cpu_power_zenergy(CPUPowerData* cpuPowerData, float& power) {
+    CPUPowerData_zenergy* powerData_zenergy = (CPUPowerData_zenergy*)cpuPowerData;
+    if (!powerData_zenergy->energyCounterFile)
+        return false;
+
+    rewind(powerData_zenergy->energyCounterFile);
+    fflush(powerData_zenergy->energyCounterFile);
+
+    uint64_t energyCounterValue = 0;
+    if (fscanf(powerData_zenergy->energyCounterFile, "%" SCNu64, &energyCounterValue) != 1)
+        return false;
+
+    Clock::time_point now = Clock::now();
+    Clock::duration timeDiff = now - powerData_zenergy->lastCounterValueTime;
+    int64_t timeDiffMicro = std::chrono::duration_cast<std::chrono::microseconds>(timeDiff).count();
+    uint64_t energyCounterDiff = energyCounterValue - powerData_zenergy->lastCounterValue;
+
+
+    if (powerData_zenergy->lastCounterValue > 0 && energyCounterValue > powerData_zenergy->lastCounterValue)
+        power = (float) energyCounterDiff / (float) timeDiffMicro;
+
+    powerData_zenergy->lastCounterValue = energyCounterValue;
+    powerData_zenergy->lastCounterValueTime = now;
+
+    return true;
+}
+
 static bool get_cpu_power_rapl(CPUPowerData* cpuPowerData, float& power) {
     CPUPowerData_rapl* powerData_rapl = (CPUPowerData_rapl*)cpuPowerData;
 
@@ -373,6 +436,9 @@ bool CPUStats::UpdateCpuPower() {
         case CPU_POWER_ZENPOWER:
             if (!get_cpu_power_zenpower(m_cpuPowerData.get(), power)) return false;
             break;
+        case CPU_POWER_ZENERGY:
+            if (!get_cpu_power_zenergy(m_cpuPowerData.get(), power)) return false;
+            break;
         case CPU_POWER_RAPL:
             if (!get_cpu_power_rapl(m_cpuPowerData.get(), power)) return false;
             break;
@@ -403,7 +469,8 @@ static bool find_input(const std::string& path, const char* input_prefix, std::s
         if (uscore != std::string::npos) {
             file.erase(uscore, std::string::npos);
             input = path + "/" + file + "_input";
-            return true;
+            //9 characters should not overflow the 32-bit int
+            return std::stoi(read_line(input).substr(0, 9)) > 0;
         }
     }
     return false;
@@ -453,6 +520,14 @@ bool CPUStats::GetCpuFile() {
         } else if (name == "it8603") {
             find_input(path, "temp", input, "temp1");
             break;
+        } else if (starts_with(name, "nct")) {
+            // Only break if nct module has TSI0_TEMP node
+            if (find_input(path, "temp", input, "TSI0_TEMP"))
+                break;
+
+        } else if (name == "asusec") {
+            find_input(path, "temp", input, "CPU");
+            break;
         } else {
             path.clear();
         }
@@ -472,6 +547,15 @@ static CPUPowerData_k10temp* init_cpu_power_data_k10temp(const std::string path)
 
     std::string coreVoltageInput, coreCurrentInput;
     std::string socVoltageInput, socCurrentInput;
+    std::string socPowerInput, corePowerInput;
+
+    if(find_input(path, "power", corePowerInput, "Pcore") && find_input(path, "power", socPowerInput, "Psoc")) {
+        powerData->corePowerFile = fopen(corePowerInput.c_str(), "r");
+        powerData->socPowerFile = fopen(socPowerInput.c_str(), "r");
+        SPDLOG_DEBUG("hwmon: using input: {}", corePowerInput);
+        SPDLOG_DEBUG("hwmon: using input: {}", socPowerInput);
+        return powerData.release();
+    }
 
     if(!find_input(path, "in", coreVoltageInput, "Vcore")) return nullptr;
     if(!find_input(path, "curr", coreCurrentInput, "Icore")) return nullptr;
@@ -508,6 +592,18 @@ static CPUPowerData_zenpower* init_cpu_power_data_zenpower(const std::string pat
     return powerData.release();
 }
 
+static CPUPowerData_zenergy* init_cpu_power_data_zenergy(const std::string path) {
+    auto powerData = std::make_unique<CPUPowerData_zenergy>();
+    std::string energyCounterPath;
+
+    if(!find_input(path, "energy", energyCounterPath, "Esocket0")) return nullptr;
+
+    SPDLOG_DEBUG("hwmon: using input: {}", energyCounterPath);
+    powerData->energyCounterFile = fopen(energyCounterPath.c_str(), "r");
+
+    return powerData.release();
+}
+
 static CPUPowerData_rapl* init_cpu_power_data_rapl(const std::string path) {
     auto powerData = std::make_unique<CPUPowerData_rapl>();
 
@@ -537,10 +633,12 @@ bool CPUStats::InitCpuPowerData() {
 
         if (name == "k10temp") {
             cpuPowerData = (CPUPowerData*)init_cpu_power_data_k10temp(path);
-            break;
         } else if (name == "zenpower") {
             cpuPowerData = (CPUPowerData*)init_cpu_power_data_zenpower(path);
             break;
+        } else if (name == "zenergy") {
+            cpuPowerData = (CPUPowerData*)init_cpu_power_data_zenergy(path);
+            break;
         } else if (name == "coretemp") {
             intel = true;
         }
