@@ -4,7 +4,7 @@
 
 #include <string>
 #include <vector>
-
+#include <regex>
 enum LS_FLAGS
 {
     LS_DIRS = 0x01,
@@ -24,5 +24,6 @@ std::string get_home_dir();
 std::string get_data_dir();
 std::string get_config_dir();
 bool lib_loaded(const std::string& lib);
+std::string remove_parentheses(const std::string&);
 
 #endif //MANGOHUD_FILE_UTILS_H
