@@ -8,6 +8,9 @@
 #include <unistd.h>
 #include <spdlog/spdlog.h>
 #include <imgui.h>
+#ifdef __linux__
+#include <implot.h>
+#endif
 #include "gl_hud.h"
 #include "file_utils.h"
 #include "notify.h"
@@ -16,6 +19,11 @@
 #include <glad/glad.h>
 
 
+#define GLX_RENDERER_VENDOR_ID_MESA                      0x8183
+#define GLX_RENDERER_DEVICE_ID_MESA                      0x8184
+
+bool glx_mesa_queryInteger(int attrib, unsigned int *value);
+
 namespace MangoHud { namespace GL {
 
 struct GLVec
@@ -67,7 +75,10 @@ void imgui_init()
         return;
 
     init_spdlog();
-    parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"));
+    if (is_blacklisted())
+        return;
+
+    parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"), false);
     _params = &params;
 
    //check for blacklist item in the config file
@@ -93,7 +104,7 @@ void imgui_init()
 }
 
 //static
-void imgui_create(void *ctx)
+void imgui_create(void *ctx, const gl_wsi plat)
 {
     if (inited)
         return;
@@ -104,21 +115,20 @@ void imgui_create(void *ctx)
     imgui_shutdown();
     imgui_init();
     inited = true;
-    // if using vulkan skip OpenGL impl, fixes issues with ZINK and multiple games using DXVK
-    if (lib_loaded("libvulkan.so") && (sw_stats.engine == ZINK || sw_stats.engine == WINED3D)) {
-        SPDLOG_INFO("libvulkan is loaded, skipping OPENGL");
-        return;
-    }
 
     if (!gladLoadGL())
         spdlog::error("Failed to initialize OpenGL context, crash incoming");
 
+    deviceName = (char*)glGetString(GL_RENDERER);
+    // If we're running zink we want to rely on the vulkan loader for the hud instead.
+    if (deviceName.find("zink") != std::string::npos)
+        return;
+
     GetOpenGLVersion(sw_stats.version_gl.major,
         sw_stats.version_gl.minor,
         sw_stats.version_gl.is_gles);
 
     std::string vendor = (char*)glGetString(GL_VENDOR);
-    deviceName = (char*)glGetString(GL_RENDERER);
     SPDLOG_DEBUG("vendor: {}, deviceName: {}", vendor, deviceName);
     sw_stats.deviceName = deviceName;
     if (vendor.find("AMD") != std::string::npos
@@ -129,18 +139,29 @@ void imgui_create(void *ctx)
     } else if (vendor.find("Intel") != std::string::npos
     || deviceName.find("Intel") != std::string::npos) {
         vendorID = 0x8086;
+    } else if (vendor.find("freedreno") != std::string::npos) {
+        vendorID = 0x5143;
     }  else {
         vendorID = 0x10de;
     }
-    if (deviceName.find("zink") != std::string::npos)
-        sw_stats.engine = EngineTypes::ZINK;
-    init_gpu_stats(vendorID, 0, params);
-    sw_stats.gpuName = gpu = get_device_name(vendorID, deviceID);
+
+    HUDElements.vendorID = vendorID;
+
+    uint32_t device_id = 0;
+    if (plat == gl_wsi::GL_WSI_GLX)
+        glx_mesa_queryInteger(GLX_RENDERER_DEVICE_ID_MESA, &device_id);
+
+    SPDLOG_DEBUG("GL device id: {:04X}", device_id);
+    init_gpu_stats(vendorID, device_id, params);
+    sw_stats.gpuName = gpu = remove_parentheses(deviceName);
     SPDLOG_DEBUG("gpu: {}", gpu);
     // Setup Dear ImGui context
     IMGUI_CHECKVERSION();
     ImGuiContext *saved_ctx = ImGui::GetCurrentContext();
     state.imgui_ctx = ImGui::CreateContext();
+#ifdef __linux__
+    ImPlot::CreateContext();
+#endif
     ImGuiIO& io = ImGui::GetIO(); (void)io;
     //io.ConfigFlags |= ImGuiConfigFlags_NavEnableKeyboard;     // Enable Keyboard Controls
     //io.ConfigFlags |= ImGuiConfigFlags_NavEnableGamepad;      // Enable Gamepad Controls
@@ -176,11 +197,11 @@ void imgui_shutdown()
     inited = false;
 }
 
-void imgui_set_context(void *ctx)
+void imgui_set_context(void *ctx, const gl_wsi plat)
 {
     if (!ctx)
         return;
-    imgui_create(ctx);
+    imgui_create(ctx, plat);
 }
 
 void imgui_render(unsigned int width, unsigned int height)
