@@ -7,11 +7,18 @@
 
 namespace MangoHud { namespace GL {
 
+enum gl_wsi
+{
+    GL_WSI_UNKNOWN,
+    GL_WSI_GLX,
+    GL_WSI_EGL,
+};
+
 extern overlay_params params;
 void imgui_init();
-void imgui_create(void *ctx);
+void imgui_create(void *ctx, const gl_wsi plat);
 void imgui_shutdown();
-void imgui_set_context(void *ctx);
+void imgui_set_context(void *ctx, const gl_wsi plat);
 void imgui_render(unsigned int width, unsigned int height);
 
 }} // namespace
