@@ -19,6 +19,7 @@
 
 #include <glad/glad.h>
 #include "gl_hud.h"
+#include "../config.h"
 
 using namespace MangoHud::GL;
 
@@ -51,6 +52,17 @@ static void* get_glx_proc_address(const char* name) {
     return func;
 }
 
+bool glx_mesa_queryInteger(int attrib, unsigned int *value);
+bool glx_mesa_queryInteger(int attrib, unsigned int *value)
+{
+    static int (*pfn_queryInteger)(int attribute, unsigned int *value) =
+        reinterpret_cast<decltype(pfn_queryInteger)>(get_glx_proc_address(
+                    "glXQueryCurrentRendererIntegerMESA"));
+    if (pfn_queryInteger)
+        return !!pfn_queryInteger(attrib, value);
+    return false;
+}
+
 EXPORT_C_(void *) glXCreateContext(void *dpy, void *vis, void *shareList, int direct)
 {
     glx.Load();
@@ -99,8 +111,8 @@ EXPORT_C_(int) glXMakeCurrent(void* dpy, void* drawable, void* ctx) {
 
     if (!is_blacklisted()) {
         if (ret) {
-            imgui_set_context(ctx);
-            SPDLOG_DEBUG("GL ref count: {}", refcnt);
+            imgui_set_context(ctx, gl_wsi::GL_WSI_GLX);
+            SPDLOG_DEBUG("GL ref count: {}", refcnt.load());
         }
 
         // Afaik -1 only works with EXT version if it has GLX_EXT_swap_control_tear, maybe EGL_MESA_swap_control_tear someday
@@ -123,7 +135,7 @@ static void do_imgui_swap(void *dpy, void *drawable)
 {
     GLint vp[4];
     if (!is_blacklisted()) {
-        imgui_create(glx.GetCurrentContext());
+        imgui_create(glx.GetCurrentContext(), gl_wsi::GL_WSI_GLX);
 
         unsigned int width = -1, height = -1;
 
@@ -293,7 +305,7 @@ EXPORT_C_(void *) mangohud_find_glx_ptr(const char *name)
 EXPORT_C_(void *) glXGetProcAddress(const unsigned char* procName) {
     void *real_func = get_glx_proc_address((const char*)procName);
     void *func = mangohud_find_glx_ptr( (const char*)procName );
-    SPDLOG_TRACE("{}: '{}', real: {}, fun: {}", __func__, procName, real_func, func);
+    SPDLOG_TRACE("{}: '{}', real: {}, fun: {}", __func__, reinterpret_cast<const char*>(procName), real_func, func);
 
     if (func && real_func)
         return func;
@@ -304,7 +316,7 @@ EXPORT_C_(void *) glXGetProcAddress(const unsigned char* procName) {
 EXPORT_C_(void *) glXGetProcAddressARB(const unsigned char* procName) {
     void *real_func = get_glx_proc_address((const char*)procName);
     void *func = mangohud_find_glx_ptr( (const char*)procName );
-    SPDLOG_TRACE("{}: '{}', real: {}, fun: {}", __func__, procName, real_func, func);
+    SPDLOG_TRACE("{}: '{}', real: {}, fun: {}", __func__, reinterpret_cast<const char*>(procName), real_func, func);
     if (func && real_func)
         return func;
 
