@@ -13,10 +13,14 @@ struct amdgpu_files
     /* The following can be NULL, in that case we're using the gpu_metrics node */
     FILE *busy;
     FILE *temp;
+    FILE *junction_temp;
+    FILE *memory_temp;
     FILE *core_clock;
     FILE *memory_clock;
     FILE *power_usage;
     FILE *gtt_used;
+    FILE *fan;
+    FILE *gpu_voltage_soc;
 };
 
 extern amdgpu_files amdgpu;
@@ -24,6 +28,8 @@ extern amdgpu_files amdgpu;
 struct gpuInfo{
     int load;
     int temp;
+    int junction_temp {-1};
+    int memory_temp {-1};
     float memoryUsed;
     float memoryTotal;
     int MemClock;
@@ -36,6 +42,9 @@ struct gpuInfo{
     bool is_temp_throttled;
     bool is_other_throttled;
     float gtt_used;
+    int fan_speed;
+    int voltage;
+    bool fan_rpm;
 };
 
 extern struct gpuInfo gpu_info;
