@@ -1,14 +1,22 @@
 #include <cstdint>
+#include <cstring>
+#include <array>
+#include <algorithm>
+#include <unistd.h>
 #include "overlay.h"
 #include "timing.hpp"
 #include "logging.h"
 #include "keybinds.h"
+#include "fps_metrics.h"
+
+Clock::time_point last_f2_press, toggle_fps_limit_press, toggle_preset_press, last_f12_press, reload_cfg_press, last_upload_press;
 
 void check_keybinds(struct overlay_params& params, uint32_t vendorID){
    using namespace std::chrono_literals;
    auto now = Clock::now(); /* us */
    auto elapsedF2 = now - last_f2_press;
    auto elapsedFpsLimitToggle = now - toggle_fps_limit_press;
+   auto elapsedPresetToggle = now - toggle_preset_press;
    auto elapsedF12 = now - last_f12_press;
    auto elapsedReloadCfg = now - reload_cfg_press;
    auto elapsedUpload = now - last_upload_press;
@@ -18,7 +26,7 @@ void check_keybinds(struct overlay_params& params, uint32_t vendorID){
       return;
    last_check = now;
 
-   auto keyPressDelay = 400ms;
+   const auto keyPressDelay = 400ms;
 
    if (elapsedF2 >= keyPressDelay &&
        keys_are_pressed(params.toggle_logging)) {
@@ -50,6 +58,19 @@ void check_keybinds(struct overlay_params& params, uint32_t vendorID){
       }
    }
 
+   if (elapsedPresetToggle >= keyPressDelay &&
+       keys_are_pressed(params.toggle_preset)) {
+     toggle_preset_press = now;
+     size_t size = params.preset.size();
+     for (size_t i = 0; i < size; i++){
+       if(params.preset[i] == current_preset) {
+         current_preset = params.preset[++i%size];
+         parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"), true);
+         break;
+       }
+     }
+   }
+
    if (elapsedF12 >= keyPressDelay &&
        keys_are_pressed(params.toggle_hud)) {
       last_f12_press = now;
@@ -58,7 +79,7 @@ void check_keybinds(struct overlay_params& params, uint32_t vendorID){
 
    if (elapsedReloadCfg >= keyPressDelay &&
        keys_are_pressed(params.reload_cfg)) {
-      parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"));
+      parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"), false);
       _params = &params;
       reload_cfg_press = now;
    }
@@ -80,4 +101,11 @@ void check_keybinds(struct overlay_params& params, uint32_t vendorID){
       next_hud_position(params);
       last_f12_press = now;
    }
+
+   if (elapsedF12 >= keyPressDelay &&
+       keys_are_pressed(params.reset_fps_metrics)) {
+      last_f12_press = now;
+      if (fpsmetrics)
+         fpsmetrics->reset_metrics();
+   }
 }
