@@ -16,6 +16,7 @@ class libx11_loader {
 
   decltype(&::XOpenDisplay) XOpenDisplay;
   decltype(&::XCloseDisplay) XCloseDisplay;
+  decltype(&::XDefaultScreen) XDefaultScreen;
   decltype(&::XQueryKeymap) XQueryKeymap;
   decltype(&::XKeysymToKeycode) XKeysymToKeycode;
   decltype(&::XStringToKeysym) XStringToKeysym;
@@ -33,4 +34,4 @@ class libx11_loader {
   void operator=(const libx11_loader&);
 };
 
-extern std::shared_ptr<libx11_loader> g_x11;
+std::shared_ptr<libx11_loader> get_libx11();
