@@ -1,6 +1,7 @@
 #include <sstream>
 #include <iomanip>
 #include <array>
+#include <algorithm>
 #include <spdlog/spdlog.h>
 #include "logging.h"
 #include "overlay.h"
@@ -26,7 +27,8 @@ string exec(string command) {
 #endif
     std::array<char, 128> buffer;
     std::string result;
-    std::unique_ptr<FILE, decltype(&pclose)> pipe(popen(command.c_str(), "r"), pclose);
+    auto deleter = [](FILE* ptr){ pclose(ptr); };
+    std::unique_ptr<FILE, decltype(deleter)> pipe(popen(command.c_str(), "r"), deleter);
     if (!pipe) {
       return "popen failed!";
     }
@@ -42,6 +44,7 @@ static void upload_file(std::string logFile){
 
   command += " | grep Location | cut -c11-";
   std::string url = exec(command);
+  std::cout << "upload url: " << url;
   exec("xdg-open " + url);
 }
 
@@ -52,6 +55,7 @@ static void upload_files(const std::vector<std::string>& logFiles){
 
   command += " | grep Location | cut -c11-";
   std::string url = exec(command);
+  std::cout << "upload url: " << url;
   exec("xdg-open " + url);
 }
 
@@ -68,41 +72,98 @@ static void writeSummary(string filename){
   SPDLOG_DEBUG("Writing summary log file [{}]", filename);
   std::ofstream out(filename, ios::out | ios::app);
   if (out){
-    out << "0.1% Min FPS," << "1% Min FPS," << "97% Percentile FPS," << "Average FPS," << "GPU Load," << "CPU Load" << "\n";
+    out << "0.1% Min FPS," << "1% Min FPS," << "97% Percentile FPS," << "Average FPS," << "GPU Load," << "CPU Load," << "Average Frame Time," << "Average GPU Temp," << "Average CPU Temp," << "Average VRAM Used," << "Average RAM Used," << "Average Swap Used," << "Peak GPU Load," << "Peak CPU Load," << "Peak GPU Temp," << "Peak CPU Temp," << "Peak VRAM Used," << "Peak RAM Used," << "Peak Swap Used" << "\n";
     std::vector<logData> sorted = logArray;
     std::sort(sorted.begin(), sorted.end(), compareByFps);
     float total = 0.0f;
-    float total_cpu = 0.0f;
     float total_gpu = 0.0f;
+    float total_cpu = 0.0f;
+    int total_gpu_temp = 0.0f;
+    int total_cpu_temp = 0.0f;
+    float total_vram = 0.0f;
+    float total_ram = 0.0f;
+    float total_swap = 0.0f;
+    int peak_gpu = 0.0f;
+    float peak_cpu = 0.0f;
+    int peak_gpu_temp = 0.0f;
+    int peak_cpu_temp = 0.0f;
+    float peak_vram = 0.0f;
+    float peak_ram = 0.0f;
+    float peak_swap = 0.0f;
     float result;
     float percents[2] = {0.001, 0.01};
     for (auto percent : percents){
       total = 0;
       size_t idx = ceil(sorted.size() * percent);
       for (size_t i = 0; i < idx; i++){
-        total = total + sorted[i].fps;
+        total = total + sorted[i].frametime;
       }
-      result = total / idx;
+      result = 1000 / (total / idx);
       out << fixed << setprecision(1) << result << ",";
     }
     // 97th percentile
-    result = sorted.empty() ? 0.0f : sorted[floor(0.97 * (sorted.size() - 1))].fps;
+    result = sorted.empty() ? 0.0f : 1000 / sorted[floor(0.97 * (sorted.size() - 1))].frametime;
     out << fixed << setprecision(1) << result << ",";
-    // avg
+    // avg + peak
     total = 0;
     for (auto input : sorted){
-      total = total + input.fps;
-      total_cpu = total_cpu + input.cpu_load;
+      total = total + input.frametime;
       total_gpu = total_gpu + input.gpu_load;
+      total_cpu = total_cpu + input.cpu_load;
+      total_gpu_temp = total_gpu_temp + input.gpu_temp;
+      total_cpu_temp = total_cpu_temp + input.cpu_temp;
+      total_vram = total_vram + input.gpu_vram_used;
+      total_ram = total_ram + input.ram_used;
+      total_swap = total_swap + input.swap_used;
+      peak_gpu = std::max(peak_gpu, input.gpu_load);
+      peak_cpu = std::max(peak_cpu, input.cpu_load);
+      peak_gpu_temp = std::max(peak_gpu_temp, input.gpu_temp);
+      peak_cpu_temp = std::max(peak_cpu_temp, input.cpu_temp);
+      peak_vram = std::max(peak_vram, input.gpu_vram_used);
+      peak_ram = std::max(peak_ram, input.ram_used);
+      peak_swap = std::max(peak_swap, input.swap_used);
     }
-    result = total / sorted.size();
+    // Average FPS
+    result = 1000 / (total / sorted.size());
     out << fixed << setprecision(1) << result << ",";
-    // GPU
+    // GPU Load (Average)
     result = total_gpu / sorted.size();
     out << result << ",";
-    // CPU
+    // CPU Load (Average)
     result = total_cpu / sorted.size();
-    out << result;
+    out << result << ",";
+    // Average Frame Time
+    result = total / sorted.size();
+    out << result << ",";
+    // Average GPU Temp
+    result = total_gpu_temp / sorted.size();
+    out << result << ",";
+    // Average CPU Temp
+    result = total_cpu_temp / sorted.size();
+    out << result << ",";
+    // Average VRAM Used
+    result = total_vram / sorted.size();
+    out << result << ",";
+    // Average RAM Used
+    result = total_ram / sorted.size();
+    out << result << ",";
+    // Average Swap Used
+    result = total_swap / sorted.size();
+    out << result << ",";
+    // Peak GPU Load
+    out << peak_gpu << ",";
+    // Peak CPU Load
+    out << peak_cpu << ",";
+    // Peak GPU Temp
+    out << peak_gpu_temp << ",";
+    // Peak CPU Temp
+    out << peak_cpu_temp << ",";
+    // Peak VRAM Used
+    out << peak_vram << ",";
+    // Peak RAM Used
+    out << peak_ram << ",";
+    // Peak Swap Used
+    out << peak_swap;
   } else {
     SPDLOG_ERROR("Failed to write log file");
   }
@@ -123,7 +184,8 @@ static void writeFileHeaders(ofstream& out){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_log_versioning])
       out << "--------------------FRAME METRICS--------------------" << endl;
 
-    out << "fps," << "frametime," << "cpu_load," << "gpu_load," << "cpu_temp," << "gpu_temp," << "gpu_core_clock," << "gpu_mem_clock," << "gpu_vram_used," << "gpu_power," << "ram_used," << "elapsed" << endl;
+    out << "fps," << "frametime," << "cpu_load," << "gpu_load," << "cpu_temp," << "gpu_temp," << "gpu_core_clock," << "gpu_mem_clock," << "gpu_vram_used," << "gpu_power," << "ram_used," << "swap_used," << "process_rss," << "elapsed" << endl;
+
 }
 
 void Logger::writeToFile(){
@@ -145,6 +207,8 @@ void Logger::writeToFile(){
     output_file << logArray.back().gpu_vram_used << ",";
     output_file << logArray.back().gpu_power << ",";
     output_file << logArray.back().ram_used << ",";
+    output_file << logArray.back().swap_used << ",";
+    output_file << logArray.back().process_rss << ",";
     output_file << std::chrono::duration_cast<std::chrono::nanoseconds>(logArray.back().previous).count() << "\n";
     output_file.flush();
   } else {
@@ -180,15 +244,11 @@ void Logger::start_logging() {
   m_log_start = Clock::now();
 
   std::string program = get_wine_exe_name();
-  printf("%s\n", output_folder.c_str());
+
   if (program.empty())
       program = get_program_name();
 
-#ifdef MANGOAPP
-  m_log_files.emplace_back(output_folder + "/mangoapp_" + program + "_" + get_log_suffix());
-#else
   m_log_files.emplace_back(output_folder + "/" + program + "_" + get_log_suffix());
-#endif
 
   if(log_interval != 0){
     std::thread log_thread(&Logger::logging, this);
@@ -269,14 +329,16 @@ void autostart_log(int sleep) {
 void Logger::calculate_benchmark_data(){
   vector<float> sorted {};
   for (auto& point : m_log_array)
-    sorted.push_back(point.fps);
+    sorted.push_back(point.frametime);
 
-  std::sort(sorted.begin(), sorted.end());
+  std::sort(sorted.begin(), sorted.end(), [](float a, float b) {
+    return a > b;
+  });
   benchmark.percentile_data.clear();
 
   benchmark.total = 0.f;
-  for (auto fps_ : sorted){
-    benchmark.total = benchmark.total + fps_;
+  for (auto frametime_ : sorted){
+    benchmark.total = benchmark.total + frametime_;
   }
 
   size_t max_label_size = 0;
@@ -297,17 +359,17 @@ void Logger::calculate_benchmark_data(){
       if (percentile.length() > max_label_size)
         max_label_size = percentile.length();
 
-      benchmark.percentile_data.push_back({percentile, result});
+      benchmark.percentile_data.push_back({percentile, (1000 / result)});
   }
   string label;
-  float mins[2] = {0.01f, 0.001f}, total;
+  float mins[2] = {0.01f, 0.001f};
   for (auto percent : mins){
-    total = 0;
-    size_t idx = ceil(sorted.size() * percent);
-    for (size_t i = 0; i < idx; i++){
-      total = total + sorted[i];
-    }
-    result = total / idx;
+    if (sorted.empty())
+      continue;
+
+    size_t percentile_pos = sorted.size() * percent;
+    percentile_pos = std::min(percentile_pos, sorted.size() - 1);
+    float result = 1000 / sorted[percentile_pos];
 
     if (percent == 0.001f)
       label = "0.1%";
