@@ -16,20 +16,23 @@ static std::unique_ptr<Display, std::function<void(Display*)>> display;
 
 struct nvctrlInfo nvctrl_info;
 bool nvctrlSuccess = false;
+int num_coolers = 0;
 
 static bool find_nv_x11(libnvctrl_loader& nvctrl, Display*& dpy)
 {
     char buf[8] {};
+    auto libx11 = get_libx11();
     for (int i = 0; i < 16; i++) {
         snprintf(buf, sizeof(buf), ":%d", i);
-        Display *d = g_x11->XOpenDisplay(buf);
+        Display *d = libx11->XOpenDisplay(buf);
         if (d) {
-            if (nvctrl.XNVCTRLIsNvScreen(d, 0)) {
+            int s = libx11->XDefaultScreen(d);
+            if (nvctrl.XNVCTRLIsNvScreen(d, s)) {
                 dpy = d;
                 SPDLOG_DEBUG("XNVCtrl is using display {}", buf);
                 return true;
             }
-            g_x11->XCloseDisplay(d);
+            libx11->XCloseDisplay(d);
         }
     }
     return false;
@@ -37,12 +40,12 @@ static bool find_nv_x11(libnvctrl_loader& nvctrl, Display*& dpy)
 
 bool checkXNVCtrl()
 {
-    if (!g_x11->IsLoaded())
+    if (!get_libx11()->IsLoaded())
         return false;
 
     auto& nvctrl = get_libnvctrl_loader();
     if (!nvctrl.IsLoaded()) {
-        SPDLOG_ERROR("XNVCtrl loader failed to load");
+        SPDLOG_DEBUG("XNVCtrl loader failed to load");
         return false;
     }
 
@@ -54,7 +57,7 @@ bool checkXNVCtrl()
         return false;
     }
 
-    auto local_x11 = g_x11;
+    auto local_x11 = get_libx11();
     display = { dpy,
         [local_x11](Display *dpy) {
             local_x11->XCloseDisplay(dpy);
@@ -70,6 +73,11 @@ bool checkXNVCtrl()
                     &pci_id);
     deviceID = (pci_id & 0xFFFF);
 
+    // get number of coolers at init
+    nvctrl.XNVCTRLQueryTargetCount(display.get(),
+                    NV_CTRL_TARGET_TYPE_COOLER,
+                    &num_coolers);
+
     return true;
 }
 
@@ -156,4 +164,20 @@ void getNvctrlInfo(){
                         NV_CTRL_USED_DEDICATED_GPU_MEMORY,
                         &memused);
     nvctrl_info.memoryUsed = memused;
+
+    nvctrl_info.fan_speed = getNvctrlFanSpeed();
+}
+
+int64_t getNvctrlFanSpeed(){
+    int64_t fan_speed = 0;
+    if (num_coolers >= 1) {
+        auto& nvctrl = get_libnvctrl_loader();
+        nvctrl.XNVCTRLQueryTargetAttribute64(display.get(),
+                            NV_CTRL_TARGET_TYPE_COOLER,
+                            0,
+                            0,
+                            NV_CTRL_THERMAL_COOLER_SPEED,
+                            &fan_speed);
+    }
+    return fan_speed;
 }
