@@ -9,11 +9,13 @@ struct nvctrlInfo{
     float memoryTotal;
     int MemClock;
     int CoreClock;
+    int fan_speed;
 };
 
 extern struct nvctrlInfo nvctrl_info;
 extern bool nvctrlSuccess;
 bool checkXNVCtrl(void);
 void getNvctrlInfo(void);
+int64_t getNvctrlFanSpeed();
 
 #endif //MANGOHUD_NVCTRL_H
