@@ -8,7 +8,7 @@
 #include <spdlog/sinks/stdout_color_sinks.h>
 #include <spdlog/sinks/rotating_file_sink.h>
 #include <filesystem.h>
-#include <sys/stat.h>
+// #include <sys/stat.h>
 #include "overlay.h"
 #include "cpu.h"
 #include "gpu.h"
@@ -17,13 +17,16 @@
 #include "fcat.h"
 #include "mesa/util/macros.h"
 #include "battery.h"
-#include "gamepad.h"
+#include "device.h"
 #include "string_utils.h"
 #include "file_utils.h"
 #include "pci_ids.h"
 #include "iostats.h"
 #include "amdgpu.h"
-
+#include "fps_metrics.h"
+#include "intel.h"
+#include "msm.h"
+#include "net.h"
 
 #ifdef __linux__
 #include <libgen.h>
@@ -42,6 +45,7 @@ struct fps_limit fps_limit_stats {};
 ImVec2 real_font_size;
 std::deque<logData> graph_data;
 const char* engines[] = {"Unknown", "OpenGL", "VULKAN", "DXVK", "VKD3D", "DAMAVAND", "ZINK", "WINED3D", "Feral3D", "ToGL", "GAMESCOPE"};
+const char* engines_short[] = {"Unknown", "OGL", "VK", "DXVK", "VKD3D", "DV", "ZINK", "WD3D", "Feral3D", "ToGL", "GS"};
 overlay_params *_params {};
 double min_frametime, max_frametime;
 bool gpu_metrics_exists = false;
@@ -49,6 +53,8 @@ bool steam_focused = false;
 vector<float> frametime_data(200,0.f);
 int fan_speed;
 fcatoverlay fcatstatus;
+std::string drm_dev;
+int current_preset;
 
 void init_spdlog()
 {
@@ -69,7 +75,7 @@ void init_spdlog()
          SPDLOG_ERROR("{}", ex.what());
       }
    }
-#ifndef NDEBUG
+#ifdef DEBUG
    spdlog::set_level(spdlog::level::level_enum::debug);
 #endif
    spdlog::cfg::load_env_levels();
@@ -85,6 +91,12 @@ void init_spdlog()
             spdlog::set_level(spdlog::level::from_str(log_level));
          }
       }
+#ifndef DEBUG
+   } else {
+      std::string log_level = "err";
+      transform(log_level.begin(), log_level.end(), log_level.begin(), ::tolower);
+      spdlog::set_level(spdlog::level::from_str(log_level));
+#endif
    }
 
 }
@@ -120,22 +132,25 @@ void update_hw_info(const struct overlay_params& params, uint32_t vendorID)
          getAmdGpuInfo();
 #ifdef __linux__
       if (gpu_metrics_exists)
-         amdgpu_get_metrics();
+         amdgpu_get_metrics(deviceID);
 #endif
       if (vendorID == 0x10de)
          getNvidiaGpuInfo(params);
-
+#ifdef __linux__
       if (vendorID== 0x8086)
-         getIntelGpuInfo();
+         if (intel) intel->update();
+      if (vendorID == 0x5143)
+         if (msm) msm->update();
+#endif
    }
 
 #ifdef __linux__
    if (params.enabled[OVERLAY_PARAM_ENABLED_battery])
       Battery_Stats.update();
-   if (params.enabled[OVERLAY_PARAM_ENABLED_gamepad_battery]) {
-      gamepad_update();
-      if (gamepad_found) {
-            gamepad_info();
+   if (!params.device_battery.empty()) {
+      device_update(params);
+      if (device_found) {
+            device_info();
       }
    }
    if (params.enabled[OVERLAY_PARAM_ENABLED_ram] || params.enabled[OVERLAY_PARAM_ENABLED_swap] || logger->is_active())
@@ -154,6 +169,8 @@ void update_hw_info(const struct overlay_params& params, uint32_t vendorID)
    currentLogData.gpu_power = gpu_info.powerUsage;
 #ifdef __linux__
    currentLogData.ram_used = memused;
+   currentLogData.swap_used = swapused;
+   currentLogData.process_rss = proc_mem.resident / float((2 << 29)); // GiB, consistent w/ other mem stats
 #endif
 
    currentLogData.cpu_load = cpuStats.GetCPUDataTotal().percent;
@@ -235,17 +252,27 @@ void update_hud_info_with_frametime(struct swapchain_stats& sw_stats, const stru
    if (sw_stats.last_present_time) {
         sw_stats.frames_stats[f_idx].stats[OVERLAY_PLOTS_frame_timing] =
             frametime_ns;
-      frametime_data[f_idx] = frametime_ms;
+      frametime_data.push_back(frametime_ms);
+      frametime_data.erase(frametime_data.begin());
    }
-
+#ifdef __linux__
+   if (throttling)
+      throttling->update();
+#endif
    frametime = frametime_ms;
    fps = double(1000 / frametime_ms);
+   if (fpsmetrics) fpsmetrics->update(now, fps);
 
    if (elapsed >= params.fps_sampling_period) {
       if (!hw_update_thread)
          hw_update_thread = std::make_unique<hw_info_updater>();
       hw_update_thread->update(&params, vendorID);
 
+      if (fpsmetrics) fpsmetrics->update_thread();
+#ifdef __linux__
+      if (HUDElements.net) HUDElements.net->update();
+#endif
+
       sw_stats.fps = 1000000000.0 * sw_stats.n_frames_since_update / elapsed;
 
       if (params.enabled[OVERLAY_PARAM_ENABLED_time]) {
@@ -255,6 +282,13 @@ void update_hud_info_with_frametime(struct swapchain_stats& sw_stats, const stru
          sw_stats.time = time.str();
       }
 
+      if (params.autostart_log && logger && !logger->autostart_init) {
+         if ((std::chrono::steady_clock::now() - HUDElements.overlay_start) > std::chrono::seconds(params.autostart_log)){
+            logger->start_logging();
+            logger->autostart_init = true;
+         }
+      }
+
       sw_stats.n_frames_since_update = 0;
       sw_stats.last_fps_update = now;
 
@@ -354,11 +388,21 @@ void position_layer(struct swapchain_stats& data, const struct overlay_params& p
       ImGui::SetNextWindowPos(data.main_window_pos, ImGuiCond_Always);
       break;
    case LAYER_POSITION_TOP_CENTER:
-      data.main_window_pos = ImVec2((width / 2) - (window_size.x / 2), margin + params.offset_y);
+      if (params.enabled[OVERLAY_PARAM_ENABLED_horizontal] && !params.enabled[OVERLAY_PARAM_ENABLED_horizontal_stretch]) {
+         float content_width = (params.table_columns  * 64);
+         data.main_window_pos = ImVec2((width / 2) - (window_size.x / 2) - content_width, margin + params.offset_y);
+      }
+      else
+         data.main_window_pos = ImVec2((width / 2) - (window_size.x / 2), margin + params.offset_y);
       ImGui::SetNextWindowPos(data.main_window_pos, ImGuiCond_Always);
       break;
    case LAYER_POSITION_BOTTOM_CENTER:
-      data.main_window_pos = ImVec2((width / 2) - (window_size.x / 2), height - window_size.y - margin + params.offset_y);
+      if (params.enabled[OVERLAY_PARAM_ENABLED_horizontal] && !params.enabled[OVERLAY_PARAM_ENABLED_horizontal_stretch]) {
+         float content_width = (params.table_columns  * 64);
+         data.main_window_pos = ImVec2((width / 2) - (window_size.x / 2) - content_width,  height - window_size.y - margin + params.offset_y);
+      }
+      else
+         data.main_window_pos = ImVec2((width / 2) - (window_size.x / 2), height - window_size.y - margin + params.offset_y);
       ImGui::SetNextWindowPos(data.main_window_pos, ImGuiCond_Always);
       break;
    case LAYER_POSITION_COUNT:
@@ -366,6 +410,30 @@ void position_layer(struct swapchain_stats& data, const struct overlay_params& p
    }
 }
 
+void RenderOutlinedText(const char* text, ImU32 textColor) {
+   ImGuiWindow* window = ImGui::GetCurrentWindow();
+   ImGuiContext& g = *GImGui;
+   const ImGuiStyle& style = g.Style;
+
+   float outlineThickness = HUDElements.params->text_outline_thickness;
+   ImVec2 textSize = ImGui::CalcTextSize(text);
+   ImU32 outlineColor = ImGui::ColorConvertFloat4ToU32(HUDElements.colors.text_outline);
+   ImVec2 pos = window->DC.CursorPos;
+
+   ImDrawList* drawList = ImGui::GetWindowDrawList();
+
+   if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_text_outline] && outlineThickness > 0.0f) {
+      drawList->AddText(ImGui::GetFont(), ImGui::GetFontSize(), ImVec2(pos.x - outlineThickness, pos.y), outlineColor, text);
+      drawList->AddText(ImGui::GetFont(), ImGui::GetFontSize(), ImVec2(pos.x + outlineThickness, pos.y), outlineColor, text);
+      drawList->AddText(ImGui::GetFont(), ImGui::GetFontSize(), ImVec2(pos.x, pos.y - outlineThickness), outlineColor, text);
+      drawList->AddText(ImGui::GetFont(), ImGui::GetFontSize(), ImVec2(pos.x, pos.y + outlineThickness), outlineColor, text);
+   }
+
+   drawList->AddText(ImGui::GetFont(), ImGui::GetFontSize(), pos, textColor, text);
+
+   ImGui::ItemSize(textSize, style.FramePadding.y);
+}
+
 void right_aligned_text(ImVec4& col, float off_x, const char *fmt, ...)
 {
    ImVec2 pos = ImGui::GetCursorPos();
@@ -380,8 +448,8 @@ void right_aligned_text(ImVec4& col, float off_x, const char *fmt, ...)
       ImVec2 sz = ImGui::CalcTextSize(buffer);
       ImGui::SetCursorPosX(pos.x + off_x - sz.x);
    }
-
-   ImGui::TextColored(col,"%s", buffer);
+   RenderOutlinedText(buffer, ImGui::ColorConvertFloat4ToU32(col));
+   // ImGui::TextColored(col,"%s", buffer);
 }
 
 void center_text(const std::string& text)
@@ -463,11 +531,11 @@ void render_mpris_metadata(const struct overlay_params& params, mutexed_metadata
          if (fmt.text.empty()) continue;
          new_pos = get_ticker_limited_pos(meta.ticker.pos, fmt.width, left_limit, right_limit);
          ImGui::SetCursorPosX(new_pos);
-         ImGui::TextColored(color, "%s", fmt.text.c_str());
+         HUDElements.TextColored(color, "%s", fmt.text.c_str());
       }
 
       if (!meta.meta.playing) {
-         ImGui::TextColored(color, "(paused)");
+         HUDElements.TextColored(color, "(paused)");
       }
 
       //ImGui::PopFont();
@@ -540,9 +608,9 @@ static void render_benchmark(swapchain_stats& data, const struct overlay_params&
    ImGui::PushStyleColor(ImGuiCol_FrameBg, ImVec4(0.0, 0.0, 0.0, alpha / params.background_alpha));
    ImGui::Dummy(ImVec2(0.0f, 8.0f));
    if (params.enabled[OVERLAY_PARAM_ENABLED_histogram])
-      ImGui::PlotHistogram("", benchmark.fps_data.data(), benchmark.fps_data.size(), 0, "", 0.0f, max + 10, ImVec2(ImGui::GetContentRegionAvailWidth(), 50));
+      ImGui::PlotHistogram("", benchmark.fps_data.data(), benchmark.fps_data.size(), 0, "", 0.0f, max + 10, ImVec2(ImGui::GetContentRegionAvail().x, 50));
    else
-      ImGui::PlotLines("", benchmark.fps_data.data(), benchmark.fps_data.size(), 0, "", 0.0f, max + 10, ImVec2(ImGui::GetContentRegionAvailWidth(), 50));
+      ImGui::PlotLines("", benchmark.fps_data.data(), benchmark.fps_data.size(), 0, "", 0.0f, max + 10, ImVec2(ImGui::GetContentRegionAvail().x, 50));
    ImGui::PopStyleColor(2);
    ImGui::End();
 }
@@ -567,17 +635,41 @@ ImVec4 change_on_load_temp(LOAD_DATA& data, unsigned current)
    }
 }
 
-void horizontal_separator(struct overlay_params& params){
-   ImGui::SameLine();
-   ImGui::Spacing();
-   ImGui::SameLine();
-   ImGui::GetWindowDrawList()->AddLine(ImVec2(ImGui::GetCursorScreenPos().x - 5, ImGui::GetCursorScreenPos().y + 2), ImVec2(ImGui::GetCursorScreenPos().x - 5, ImGui::GetCursorScreenPos().y + params.font_size * 0.85), params.vram_color, 2);
-   ImGui::SameLine();
-   ImGui::Spacing();
+void horizontal_separator(struct overlay_params& params) {
+    ImGui::SameLine();
+    ImGui::Spacing();
+    ImGui::SameLine();
+
+    ImDrawList* drawList = ImGui::GetWindowDrawList();
+    ImVec2 cursorPos = ImGui::GetCursorScreenPos();
+    ImVec2 startPos(cursorPos.x - 5, cursorPos.y + 2);
+    ImVec2 endPos(startPos.x, cursorPos.y + params.font_size * 0.85);
+
+    float outlineThickness = 1.0f;
+
+   if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_text_outline]){
+      // Draw the black outline
+      drawList->AddLine(ImVec2(startPos.x - outlineThickness, startPos.y), ImVec2(startPos.x - outlineThickness, endPos.y), IM_COL32_BLACK, outlineThickness + 2);
+      drawList->AddLine(ImVec2(startPos.x + outlineThickness, startPos.y), ImVec2(startPos.x + outlineThickness, endPos.y), IM_COL32_BLACK, outlineThickness + 2);
+      drawList->AddLine(ImVec2(startPos.x - outlineThickness, startPos.y - outlineThickness/2), ImVec2(startPos.x + outlineThickness, startPos.y - outlineThickness/2), IM_COL32_BLACK, outlineThickness + 2);
+      drawList->AddLine(ImVec2(startPos.x - outlineThickness, endPos.y + outlineThickness/2), ImVec2(startPos.x + outlineThickness, endPos.y + outlineThickness/2), IM_COL32_BLACK, outlineThickness + 2);
+   } else {
+      outlineThickness *= 2;
+   }
+
+    // Draw the separator line
+    drawList->AddLine(startPos, endPos, params.vram_color, outlineThickness);
+
+    ImGui::SameLine();
+    ImGui::Spacing();
 }
 
 void render_imgui(swapchain_stats& data, struct overlay_params& params, ImVec2& window_size, bool is_vulkan)
 {
+   {
+      std::unique_lock<std::mutex> lock(config_mtx);
+      config_cv.wait(lock, []{ return config_ready; });
+   }
    // data.engine = EngineTypes::GAMESCOPE;
    HUDElements.sw_stats = &data; HUDElements.params = &params;
    HUDElements.is_vulkan = is_vulkan;
@@ -602,21 +694,29 @@ void render_imgui(swapchain_stats& data, struct overlay_params& params, ImVec2&
    if(params.enabled[OVERLAY_PARAM_ENABLED_horizontal])
       table_flags = ImGuiTableFlags_NoClip | ImGuiTableFlags_SizingFixedFit | ImGuiTableFlags_NoHostExtendX;
 
-   if (!params.no_display && !steam_focused){
+   if (!params.no_display && !steam_focused && params.table_columns){
       ImGui::Begin("Main", &gui_open, ImGuiWindowFlags_NoDecoration);
       if (ImGui::BeginTable("hud", params.table_columns, table_flags )) {
          HUDElements.place = 0;
          for (auto& func : HUDElements.ordered_functions){
-            ImGui::PushStyleVar(ImGuiStyleVar_CellPadding, ImVec2(-3,-3));
-            if(!params.enabled[OVERLAY_PARAM_ENABLED_horizontal])
+            if(!params.enabled[OVERLAY_PARAM_ENABLED_horizontal] && func.name != "exec")
                ImGui::TableNextRow();
-            func.first();
+            func.run();
             HUDElements.place += 1;
-            ImGui::PopStyleVar();
-            if(!HUDElements.ordered_functions.empty() && params.enabled[OVERLAY_PARAM_ENABLED_horizontal] && func != HUDElements.ordered_functions.back())
+            if(!HUDElements.ordered_functions.empty() && params.enabled[OVERLAY_PARAM_ENABLED_horizontal] && HUDElements.ordered_functions.size() != (size_t)HUDElements.place)
                horizontal_separator(params);
          }
+
+         if (params.enabled[OVERLAY_PARAM_ENABLED_horizontal]) {
+            if (HUDElements.table_columns_count > 0 && HUDElements.table_columns_count < 65 )
+               params.table_columns = HUDElements.table_columns_count;
+            if(!params.enabled[OVERLAY_PARAM_ENABLED_horizontal_stretch]) {
+               float content_width = ImGui::GetContentRegionAvail().x - (params.table_columns * 64);
+               window_size = ImVec2(content_width, params.height);
+            }
+         }
          ImGui::EndTable();
+         HUDElements.table_columns_count = 0;
       }
 
       if(logger->is_active())
@@ -698,16 +798,67 @@ void init_gpu_stats(uint32_t& vendorID, uint32_t reported_deviceID, overlay_para
       }
    }
 
-   // NVIDIA or Intel but maybe has Optimus
-   if (vendorID == 0x8086
-      || vendorID == 0x10de) {
-
+#ifdef __linux__
+   // NVIDIA
+   if (vendorID == 0x10de)
       if(checkNvidia(pci_dev))
          vendorID = 0x10de;
+
+   string path;
+   string drm = "/sys/class/drm/";
+
+   if (vendorID==0x8086){
+      auto dirs = ls(drm.c_str(), "card");
+      for (auto& dir : dirs) {
+         if (dir.find("-") != std::string::npos)
+             continue; // filter display adapters
+
+         FILE *fp;
+         string device = path + "/device/device";
+         if ((fp = fopen(device.c_str(), "r"))){
+            uint32_t temp = 0;
+            if (fscanf(fp, "%x", &temp) == 1) {
+               if (temp != reported_deviceID){
+                  fclose(fp);
+                  SPDLOG_DEBUG("DeviceID does not match vulkan report {:X}", reported_deviceID);
+                  continue;
+               }
+               deviceID = temp;
+            }
+            fclose(fp);
+         }
+
+         string vendor = path + "/device/vendor";
+         if ((fp = fopen(vendor.c_str(), "r"))){
+            uint32_t temp = 0;
+            if (fscanf(fp, "%x", &temp) != 1 || temp != 0x8086) {
+               fclose(fp);
+               continue;
+            }
+            fclose(fp);
+         }
+         path = drm + dir;
+         drm_dev = dir;
+         SPDLOG_DEBUG("Intel: using drm device {}", drm_dev);
+         intel = std::make_unique<Intel>();
+         break;
+      }
    }
 
-#ifdef __linux__
-   if (vendorID == 0x1002 || vendorID==0x8086
+   if (vendorID == 0x5143) {
+      auto dirs = ls(drm.c_str(), "card");
+      for (auto& dir : dirs) {
+         if (dir.find("-") != std::string::npos) {
+             continue; // filter display adapters
+         }
+         path = drm + dir;
+         drm_dev = dir;
+         SPDLOG_DEBUG("msm: using drm device {}", drm_dev);
+         msm = std::make_unique<MSM>();
+      }
+   }
+
+   if (vendorID == 0x1002
        || gpu.find("Radeon") != std::string::npos
        || gpu.find("AMD") != std::string::npos) {
       string path;
@@ -758,8 +909,10 @@ void init_gpu_stats(uint32_t& vendorID, uint32_t reported_deviceID, overlay_para
          const std::string device_path = path + "/device";
          const std::string gpu_metrics_path = device_path + "/gpu_metrics";
          if (amdgpu_verify_metrics(gpu_metrics_path)) {
+            gpu_info.fan_rpm = true;
             gpu_metrics_exists = true;
             metrics_path = gpu_metrics_path;
+            throttling = std::make_unique<Throttling>();
             SPDLOG_DEBUG("Using gpu_metrics of {}", gpu_metrics_path);
          }
 
@@ -771,29 +924,42 @@ void init_gpu_stats(uint32_t& vendorID, uint32_t reported_deviceID, overlay_para
             amdgpu.gtt_used = fopen((device_path + "/mem_info_gtt_used").c_str(), "r");
 
          const std::string hwmon_path = device_path + "/hwmon/";
-         const auto dirs = ls(hwmon_path.c_str(), "hwmon", LS_DIRS);
-         for (const auto& dir : dirs)
-            if (!amdgpu.temp)
-               amdgpu.temp = fopen((hwmon_path + dir + "/temp1_input").c_str(), "r");
+         if (fs::exists(hwmon_path)){
+            const auto dirs = ls(hwmon_path.c_str(), "hwmon", LS_DIRS);
+            for (const auto& dir : dirs) {
+               if (!amdgpu.temp)
+                  amdgpu.temp = fopen((hwmon_path + dir + "/temp1_input").c_str(), "r");
+               if (!amdgpu.junction_temp)
+                  amdgpu.junction_temp = fopen((hwmon_path + dir + "/temp2_input").c_str(), "r");
+               if (!amdgpu.memory_temp)
+                  amdgpu.memory_temp = fopen((hwmon_path + dir + "/temp3_input").c_str(), "r");
+               if (!amdgpu.core_clock)
+                  amdgpu.core_clock = fopen((hwmon_path + dir + "/freq1_input").c_str(), "r");
+               if (!amdgpu.gpu_voltage_soc)
+                  amdgpu.gpu_voltage_soc = fopen((hwmon_path + dir + "/in0_input").c_str(), "r");
+            }
 
-         if (!metrics_path.empty())
-            break;
+            if (!metrics_path.empty())
+               break;
 
-         // The card output nodes - cardX-output, will point to the card node
-         // As such the actual metrics nodes will be missing.
-         amdgpu.busy = fopen((device_path + "/gpu_busy_percent").c_str(), "r");
-         if (!amdgpu.busy)
-            continue;
+            // The card output nodes - cardX-output, will point to the card node
+            // As such the actual metrics nodes will be missing.
+            amdgpu.busy = fopen((device_path + "/gpu_busy_percent").c_str(), "r");
+            if (!amdgpu.busy)
+               continue;
 
-         SPDLOG_DEBUG("using amdgpu path: {}", device_path);
+            SPDLOG_DEBUG("using amdgpu path: {}", device_path);
 
-         for (const auto& dir : dirs) {
-            if (!amdgpu.core_clock)
-               amdgpu.core_clock = fopen((hwmon_path + dir + "/freq1_input").c_str(), "r");
-            if (!amdgpu.memory_clock)
-               amdgpu.memory_clock = fopen((hwmon_path + dir + "/freq2_input").c_str(), "r");
-            if (!amdgpu.power_usage)
-               amdgpu.power_usage = fopen((hwmon_path + dir + "/power1_average").c_str(), "r");
+            for (const auto& dir : dirs) {
+               if (!amdgpu.memory_clock)
+                  amdgpu.memory_clock = fopen((hwmon_path + dir + "/freq2_input").c_str(), "r");
+               if (!amdgpu.power_usage)
+                  amdgpu.power_usage = fopen((hwmon_path + dir + "/power1_average").c_str(), "r");
+               if (!amdgpu.power_usage)
+                  amdgpu.power_usage = fopen((hwmon_path + dir + "/power1_input").c_str(), "r");
+               if (!amdgpu.fan)
+                  amdgpu.fan = fopen((hwmon_path + dir + "/fan1_input").c_str(), "r");
+            }
          }
          break;
       }
@@ -805,7 +971,7 @@ void init_gpu_stats(uint32_t& vendorID, uint32_t reported_deviceID, overlay_para
    }
 #endif
    if (!params.permit_upload)
-      SPDLOG_INFO("Uploading is disabled (permit_upload = 0)");
+      SPDLOG_DEBUG("Uploading is disabled (permit_upload = 0)");
 }
 
 void init_system_info(){
@@ -828,8 +994,8 @@ void init_system_info(){
       const char* mangohud_recursion = getenv("MANGOHUD_RECURSION");
       if (!mangohud_recursion) {
          setenv("MANGOHUD_RECURSION", "1", 1);
-         driver = exec("glxinfo -B | sed -n 's/^OpenGL version.*: \\(.*\\)/\\1/p' | sed 's/([^)]*)//g;s/  / /g'");
-         trim(driver);
+         // driver = exec("glxinfo -B | sed -n 's/^OpenGL version.*: \\(.*\\)/\\1/p' | sed 's/([^)]*)//g;s/  / /g'");
+         // trim(driver);
          unsetenv("MANGOHUD_RECURSION");
       } else {
          driver = "MangoHud glxinfo recursion detected";
@@ -907,26 +1073,6 @@ void init_system_info(){
 #endif
 }
 
-std::string get_device_name(uint32_t vendorID, uint32_t deviceID)
-{
-   string desc;
-#ifdef __linux__
-   if (pci_ids.find(vendorID) == pci_ids.end())
-      parse_pciids();
-
-   desc = pci_ids[vendorID].second[deviceID].desc;
-   size_t position = desc.find("[");
-   if (position != std::string::npos) {
-      desc = desc.substr(position);
-      string chars = "[]";
-      for (char c: chars)
-         desc.erase(remove(desc.begin(), desc.end(), c), desc.end());
-   }
-   trim(desc);
-#endif
-   return desc;
-}
-
 void update_fan(){
    // This just handles steam deck fan for now
    static bool init;
