@@ -7,6 +7,7 @@
 #include <vector>
 #include <deque>
 #include <imgui.h>
+#include "imgui_internal.h"
 #include "overlay_params.h"
 #include "hud_elements.h"
 #include "engine_types.h"
@@ -89,6 +90,8 @@ extern overlay_params *_params;
 extern double min_frametime, max_frametime;
 extern bool steam_focused;
 extern int fan_speed;
+extern int current_preset;
+extern std::vector<float> frametime_data;
 
 void init_spdlog();
 void overlay_new_frame(const struct overlay_params& params);
@@ -103,7 +106,6 @@ void init_cpu_stats(overlay_params& params);
 void check_keybinds(overlay_params& params, uint32_t vendorID);
 void init_system_info(void);
 void FpsLimiter(struct fps_limit& stats);
-std::string get_device_name(uint32_t vendorID, uint32_t deviceID);
 void create_fonts(ImFontAtlas* font_atlas, const overlay_params& params, ImFont*& small_font, ImFont*& text_font);
 void right_aligned_text(ImVec4& col, float off_x, const char *fmt, ...);
 void center_text(const std::string& text);
@@ -120,4 +122,5 @@ void render_mpris_metadata(const overlay_params& params, mutexed_metadata& meta,
 void update_fan();
 void next_hud_position(struct overlay_params& params);
 void horizontal_separator(struct overlay_params& params);
+void RenderOutlinedText(const char* text, ImU32 textColor);
 #endif //MANGOHUD_OVERLAY_H
