@@ -14,6 +14,7 @@
 #include <iostream>
 #include <string>
 #include <sstream>
+#include <fstream>
 #include <algorithm>
 #include <cctype>
 #include <array>
@@ -37,6 +38,12 @@
 #include "dbus_info.h"
 
 #include "app/mangoapp.h"
+#include "fps_metrics.h"
+
+std::unique_ptr<fpsMetrics> fpsmetrics;
+std::mutex config_mtx;
+std::condition_variable config_cv;
+bool config_ready = false;
 
 #if __cplusplus >= 201703L
 
@@ -88,6 +95,8 @@ parse_position(const char *str)
       return LAYER_POSITION_BOTTOM_RIGHT;
    if (!strcmp(str, "top-center"))
       return LAYER_POSITION_TOP_CENTER;
+   if (!strcmp(str, "bottom-center"))
+      return LAYER_POSITION_BOTTOM_CENTER;
    return LAYER_POSITION_TOP_LEFT;
 }
 
@@ -127,12 +136,12 @@ static std::vector<KeySym>
 parse_string_to_keysym_vec(const char *str)
 {
    std::vector<KeySym> keys;
-   if(g_x11->IsLoaded())
+   if(get_libx11()->IsLoaded())
    {
       auto keyStrings = str_tokenize(str);
       for (auto& ks : keyStrings) {
          trim(ks);
-         KeySym xk = g_x11->XStringToKeysym(ks.c_str());
+         KeySym xk = get_libx11()->XStringToKeysym(ks.c_str());
          if (xk)
             keys.push_back(xk);
          else
@@ -149,6 +158,8 @@ parse_string_to_keysym_vec(const char *str)
 #define parse_upload_log            parse_string_to_keysym_vec
 #define parse_upload_logs           parse_string_to_keysym_vec
 #define parse_toggle_fps_limit      parse_string_to_keysym_vec
+#define parse_toggle_preset         parse_string_to_keysym_vec
+#define parse_reset_fps_metrics     parse_string_to_keysym_vec
 
 #else
 #define parse_toggle_hud(x)            {}
@@ -158,8 +169,34 @@ parse_string_to_keysym_vec(const char *str)
 #define parse_upload_log(x)            {}
 #define parse_upload_logs(x)           {}
 #define parse_toggle_fps_limit(x)      {}
+#define parse_toggle_preset(x)         {}
+#define parse_reset_fps_metrics(x)     {}
 #endif
 
+// NOTE: This is NOT defined as an OVERLAY_PARAM and will be called manually
+static std::vector<int>
+parse_preset(const char *str)
+{
+  std::vector<int> presets;
+  auto preset_strings = str_tokenize(str);
+
+  for (auto& value : preset_strings) {
+    trim(value);
+
+    uint32_t as_int;
+    try {
+      as_int = static_cast<int>(std::stoi(value));
+    } catch (const std::invalid_argument&) {
+      SPDLOG_ERROR("invalid preset value: '{}'", value);
+      continue;
+    }
+
+    presets.push_back(as_int);
+  }
+
+  return presets;
+}
+
 static uint32_t
 parse_fps_sampling_period(const char *str)
 {
@@ -385,6 +422,20 @@ parse_gl_size_query(const char *str)
    return GL_SIZE_DRAWABLE;
 }
 
+static std::vector<std::string>
+parse_fps_metrics(const char *str){
+   std::vector<std::string> metrics;
+   auto tokens = str_tokenize(str);
+   for (auto& token : tokens) {
+      metrics.push_back(token);
+   }
+
+   fpsmetrics.release();
+   fpsmetrics = std::make_unique<fpsMetrics>(metrics);
+
+   return metrics;
+}
+
 #define parse_width(s) parse_unsigned(s)
 #define parse_height(s) parse_unsigned(s)
 #define parse_vsync(s) parse_unsigned(s)
@@ -404,6 +455,7 @@ parse_gl_size_query(const char *str)
 #define parse_font_scale_media_player(s) parse_float(s)
 #define parse_cpu_text(s) parse_str(s)
 #define parse_gpu_text(s) parse_str(s)
+#define parse_fps_text(s) parse_str(s)
 #define parse_log_interval(s) parse_unsigned(s)
 #define parse_font_size(s) parse_float(s)
 #define parse_font_size_text(s) parse_float(s)
@@ -422,7 +474,6 @@ parse_gl_size_query(const char *str)
 #define parse_fcat_screen_edge(s) parse_unsigned(s)
 #define parse_picmip(s) parse_signed(s)
 #define parse_af(s) parse_signed(s)
-#define parse_preset(s) parse_signed(s)
 
 #define parse_cpu_color(s) parse_color(s)
 #define parse_gpu_color(s) parse_color(s)
@@ -435,6 +486,7 @@ parse_gl_size_query(const char *str)
 #define parse_text_color(s) parse_color(s)
 #define parse_media_player_color(s) parse_color(s)
 #define parse_wine_color(s) parse_color(s)
+#define parse_network_color(s) parse_color(s)
 #define parse_gpu_load_color(s) parse_load_color(s)
 #define parse_cpu_load_color(s) parse_load_color(s)
 #define parse_gpu_load_value(s) parse_load_value(s)
@@ -447,6 +499,10 @@ parse_gl_size_query(const char *str)
 #define parse_battery_color(s) parse_color(s)
 #define parse_media_player_format(s) parse_str_tokenize(s, ";", false)
 #define parse_fsr_steam_sharpness(s) parse_float(s)
+#define parse_text_outline_color(s) parse_color(s)
+#define parse_text_outline_thickness(s) parse_float(s)
+#define parse_device_battery(s) parse_str_tokenize(s)
+#define parse_network(s) parse_str_tokenize(s)
 
 static bool
 parse_help(const char *str)
@@ -521,52 +577,115 @@ const char *overlay_param_names[] = {
 };
 
 static void
-parse_overlay_env(struct overlay_params *params,
-                  const char *env)
+initialize_preset(struct overlay_params *params)
 {
+   if (params->options.find("preset") != params->options.end()) {
+      auto presets = parse_preset(params->options.find("preset")->second.c_str());
+      if (!presets.empty())
+         params->preset = presets;
+   }
+   current_preset = params->preset[0];
+}
+
+static void
+set_parameters_from_options(struct overlay_params *params)
+{
+   bool read_cfg = false;
+   if (params->options.find("read_cfg") != params->options.end() && params->options.find("read_cfg")->second != "0")
+      read_cfg = true;
+
+   if (params->options.find("full") != params->options.end() && params->options.find("full")->second != "0") {
+#define OVERLAY_PARAM_BOOL(name) \
+      params->enabled[OVERLAY_PARAM_ENABLED_##name] = 1;
+#define OVERLAY_PARAM_CUSTOM(name)
+      OVERLAY_PARAMS
+      #undef OVERLAY_PARAM_BOOL
+      #undef OVERLAY_PARAM_CUSTOM
+      params->enabled[OVERLAY_PARAM_ENABLED_histogram] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_fps_only] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_battery_icon] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_mangoapp_steam] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_hide_fsr_sharpness] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_throttling_status] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_fcat] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_horizontal] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_hud_no_margin] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_log_versioning] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_hud_compact] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_exec_name] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_trilinear] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_bicubic] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_retro] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_debug] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_engine_short_names] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_dynamic_frame_timing] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit] = 0;
+      params->enabled[OVERLAY_PARAM_ENABLED_duration] = false;
+      params->enabled[OVERLAY_PARAM_ENABLED_core_bars] = false;
+      params->enabled[OVERLAY_PARAM_ENABLED_read_cfg] = read_cfg;
+      params->enabled[OVERLAY_PARAM_ENABLED_time_no_label] = false;
+      params->options.erase("full");
+   }
+   for (auto& it : params->options) {
+#define OVERLAY_PARAM_BOOL(name)                                 \
+      if (it.first == #name) {                                   \
+          params->enabled[OVERLAY_PARAM_ENABLED_##name] =        \
+          strtol(it.second.c_str(), NULL, 0);                    \
+          continue;                                              \
+       }
+#define OVERLAY_PARAM_CUSTOM(name)                               \
+      if (it.first == #name) {                                   \
+         params->name = parse_##name(it.second.c_str());         \
+      continue;                                                  \
+   }
+      OVERLAY_PARAMS
+      #undef OVERLAY_PARAM_BOOL
+      #undef OVERLAY_PARAM_CUSTOM
+      if (it.first == "preset") {
+         continue; // Handled above
+      }
+      SPDLOG_ERROR("Unknown option '{}'", it.first.c_str());
+   }
+}
+
+static void
+parse_overlay_env(struct overlay_params *params,
+                  const char *env, bool use_existing_preset)
+{
+   const char *env_start = env;
+
    uint32_t num;
    char key[256], value[256];
    while ((num = parse_string(env, key, value)) != 0) {
+      trim_char(key);
+      trim_char(value);
       env += num;
-      if (!strcmp("full", key)) {
-         bool read_cfg = params->enabled[OVERLAY_PARAM_ENABLED_read_cfg];
-#define OVERLAY_PARAM_BOOL(name) \
-         params->enabled[OVERLAY_PARAM_ENABLED_##name] = 1;
-#define OVERLAY_PARAM_CUSTOM(name)
-         OVERLAY_PARAMS
-#undef OVERLAY_PARAM_BOOL
-#undef OVERLAY_PARAM_CUSTOM
-         params->enabled[OVERLAY_PARAM_ENABLED_histogram] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_gpu_load_change] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_cpu_load_change] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_fps_only] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_fps_color_change] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_core_load_change] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_battery_icon] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_mangoapp_steam] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_hide_fsr_sharpness] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_throttling_status] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_read_cfg] = read_cfg;
-         params->enabled[OVERLAY_PARAM_ENABLED_fcat] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_horizontal] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_hud_no_margin] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_log_versioning] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_hud_compact] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_exec_name] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_trilinear] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_bicubic] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_retro] = 0;
+      if (!strcmp("preset", key)) {
+         if (!use_existing_preset) {
+            add_to_options(params, key, value);
+            initialize_preset(params);
+         }
+         break;
+      }
+   }
+
+   presets(current_preset, params);
+   env = env_start;
+
+   while ((num = parse_string(env, key, value)) != 0) {
+      trim_char(key);
+      trim_char(value);
+      env += num;
+      if (!strcmp("preset", key)) {
+         continue; // Avoid 'Unknown option' error
       }
 #define OVERLAY_PARAM_BOOL(name)                                       \
       if (!strcmp(#name, key)) {                                       \
-         params->enabled[OVERLAY_PARAM_ENABLED_##name] =               \
-            strtol(value, NULL, 0);                                    \
          add_to_options(params, key, value);                           \
          continue;                                                     \
       }
 #define OVERLAY_PARAM_CUSTOM(name)                                     \
       if (!strcmp(#name, key)) {                                       \
-         params->name = parse_##name(value);                           \
          add_to_options(params, key, value);                           \
          continue;                                                     \
       }
@@ -575,21 +694,19 @@ parse_overlay_env(struct overlay_params *params,
 #undef OVERLAY_PARAM_CUSTOM
       SPDLOG_ERROR("Unknown option '{}'", key);
    }
+   set_parameters_from_options(params);
 }
 
-void
-parse_overlay_config(struct overlay_params *params,
-                  const char *env)
-{
-   *params = {};
-
-   /* Visible by default */
+static void set_param_defaults(struct overlay_params *params){
    params->enabled[OVERLAY_PARAM_ENABLED_fps] = true;
    params->enabled[OVERLAY_PARAM_ENABLED_frame_timing] = true;
    params->enabled[OVERLAY_PARAM_ENABLED_core_load] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_core_bars] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_cpu_temp] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_cpu_power] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_gpu_temp] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_gpu_junction_temp] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_gpu_mem_temp] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_cpu_stats] = true;
    params->enabled[OVERLAY_PARAM_ENABLED_gpu_stats] = true;
    params->enabled[OVERLAY_PARAM_ENABLED_ram] = false;
@@ -598,18 +715,24 @@ parse_overlay_config(struct overlay_params *params,
    params->enabled[OVERLAY_PARAM_ENABLED_read_cfg] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_io_read] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_io_write] = false;
-   params->enabled[OVERLAY_PARAM_ENABLED_io_stats] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_wine] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_gpu_load_change] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_cpu_load_change] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_core_load_change] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_gpu_voltage] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_legacy_layout] = true;
    params->enabled[OVERLAY_PARAM_ENABLED_frametime] = true;
    params->enabled[OVERLAY_PARAM_ENABLED_fps_only] = false;
-   params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery] = false;
-   params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery_icon] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_device_battery_icon] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_throttling_status] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_fcat] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_horizontal_stretch] = true;
+   params->enabled[OVERLAY_PARAM_ENABLED_engine_short_names] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_text_outline] = true;
+   params->enabled[OVERLAY_PARAM_ENABLED_dynamic_frame_timing] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_duration] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_frame_timing_detailed] = false;
    params->fps_sampling_period = 500000000; /* 500ms */
    params->width = 0;
    params->height = 140;
@@ -635,6 +758,7 @@ parse_overlay_config(struct overlay_params *params,
    params->background_color = 0x020202;
    params->text_color = 0xffffff;
    params->media_player_color = 0xffffff;
+   params->network_color = 0xe07b85;
    params->media_player_name = "";
    params->font_scale = 1.0f;
    params->wine_color = 0xeb5b5b;
@@ -655,11 +779,30 @@ parse_overlay_config(struct overlay_params *params,
    params->fsr_steam_sharpness = -1;
    params->picmip = -17;
    params->af = -1;
-   params->preset = -1;
+   params->font_size = 24;
+   params->table_columns = 3;
+   params->text_outline_color = 0x000000;
+   params->text_outline_thickness = 1.5;
+}
+
+void
+parse_overlay_config(struct overlay_params *params,
+                  const char *env, bool use_existing_preset)
+{
+   std::vector<int> default_preset = {-1, 0, 1, 2, 3, 4};
+   *params = {
+     .preset = use_existing_preset ? params->preset : default_preset
+   };
+   set_param_defaults(params);
+   if (!use_existing_preset) {
+      current_preset = params->preset[0];
+   }
 
 #ifdef HAVE_X11
    params->toggle_hud = { XK_Shift_R, XK_F12 };
    params->toggle_hud_position = { XK_Shift_R, XK_F11 };
+   params->toggle_preset = { XK_Shift_R, XK_F10 };
+   params->reset_fps_metrics = { XK_Shift_R, XK_F9};
    params->toggle_fps_limit = { XK_Shift_L, XK_F1 };
    params->toggle_logging = { XK_Shift_L, XK_F2 };
    params->reload_cfg = { XK_Shift_L, XK_F4 };
@@ -669,16 +812,20 @@ parse_overlay_config(struct overlay_params *params,
 
 #ifdef _WIN32
    params->toggle_hud = { VK_F12 };
+   params->toggle_preset = { VK_F10 };
+   params->reset_fps_metrics = { VK_F9};
    params->toggle_fps_limit = { VK_F3 };
    params->toggle_logging = { VK_F2 };
    params->reload_cfg = { VK_F4 };
 
    #undef parse_toggle_hud
    #undef parse_toggle_fps_limit
+   #undef parse_toggle_preset
    #undef parse_toggle_logging
    #undef parse_reload_cfg
 
    #define parse_toggle_hud(x)         params->toggle_hud
+   #define parse_toggle_preset(x)      params->toggle_preset
    #define parse_toggle_fps_limit(x)   params->toggle_fps_limit
    #define parse_toggle_logging(x)     params->toggle_logging
    #define parse_reload_cfg(x)         params->reload_cfg
@@ -686,61 +833,31 @@ parse_overlay_config(struct overlay_params *params,
 
    HUDElements.ordered_functions.clear();
    HUDElements.exec_list.clear();
+   params->options.clear();
+   HUDElements.options.clear();
+
    // first pass with env var
    if (env)
-      parse_overlay_env(params, env);
+      parse_overlay_env(params, env, use_existing_preset);
 
    bool read_cfg = params->enabled[OVERLAY_PARAM_ENABLED_read_cfg];
+   bool env_contains_preset = params->options.find("preset") != params->options.end();
    if (!env || read_cfg) {
-
-      // Get config options
       parseConfigFile(*params);
 
-      if (params->options.find("preset") != params->options.end())
-         presets(stoi(params->options.find("preset")->second), params);
-
-      if (params->options.find("full") != params->options.end() && params->options.find("full")->second != "0") {
-#define OVERLAY_PARAM_BOOL(name) \
-            params->enabled[OVERLAY_PARAM_ENABLED_##name] = 1;
-#define OVERLAY_PARAM_CUSTOM(name)
-            OVERLAY_PARAMS
-#undef OVERLAY_PARAM_BOOL
-#undef OVERLAY_PARAM_CUSTOM
-         params->enabled[OVERLAY_PARAM_ENABLED_histogram] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_fps_only] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_battery_icon] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_mangoapp_steam] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_hide_fsr_sharpness] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_throttling_status] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_fcat] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_horizontal] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_hud_no_margin] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_log_versioning] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_hud_compact] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_exec_name] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_trilinear] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_bicubic] = 0;
-         params->enabled[OVERLAY_PARAM_ENABLED_retro] = 0;
-         params->options.erase("full");
-      }
-      for (auto& it : params->options) {
-#define OVERLAY_PARAM_BOOL(name)                                       \
-         if (it.first == #name) {                                      \
-            params->enabled[OVERLAY_PARAM_ENABLED_##name] =            \
-               strtol(it.second.c_str(), NULL, 0);                     \
-            continue;                                                  \
-         }
-#define OVERLAY_PARAM_CUSTOM(name)                                     \
-         if (it.first == #name) {                                      \
-            params->name = parse_##name(it.second.c_str());            \
-            continue;                                                  \
-         }
-         OVERLAY_PARAMS
-#undef OVERLAY_PARAM_BOOL
-#undef OVERLAY_PARAM_CUSTOM
-         SPDLOG_ERROR("Unknown option '{}'", it.first.c_str());
+      if (!use_existing_preset && !env_contains_preset) {
+         initialize_preset(params);
       }
 
+      // clear options since we don't want config options to appear first
+      params->options.clear();
+      HUDElements.options.clear();
+      // add preset options
+      presets(current_preset, params);
+      // potentially override preset options with config options
+      parseConfigFile(*params);
+
+      set_parameters_from_options(params);
    }
 
    // TODO decide what to do for legacy_layout=0
@@ -748,7 +865,7 @@ parse_overlay_config(struct overlay_params *params,
    if (params->enabled[OVERLAY_PARAM_ENABLED_legacy_layout] && env && read_cfg) {
       // If passing legacy_layout=0 to MANGOHUD_CONFIG anyway then clear first pass' results
       HUDElements.ordered_functions.clear();
-      parse_overlay_env(params, env);
+      parse_overlay_env(params, env, true);
    }
 
    // If fps_only param is enabled disable legacy_layout
@@ -762,7 +879,7 @@ parse_overlay_config(struct overlay_params *params,
       params->font_scale_media_player = 0.55f;
 
    // Convert from 0xRRGGBB to ImGui's format
-   std::array<unsigned *, 21> colors = {
+   std::array<unsigned *, 23> colors = {
       &params->cpu_color,
       &params->gpu_color,
       &params->vram_color,
@@ -784,6 +901,8 @@ parse_overlay_config(struct overlay_params *params,
       &params->fps_color[0],
       &params->fps_color[1],
       &params->fps_color[2],
+      &params->text_outline_color,
+      &params->network_color,
    };
 
    for (auto color : colors){
@@ -794,15 +913,8 @@ parse_overlay_config(struct overlay_params *params,
                255);
    }
 
-   if (!params->table_columns)
-      params->table_columns = 3;
-
    params->table_columns = std::max(1u, std::min(64u, params->table_columns));
 
-   if (!params->font_size) {
-      params->font_size = 24;
-   }
-
    //increase hud width if io read and write
    if (!params->width && !params->enabled[OVERLAY_PARAM_ENABLED_horizontal]) {
       params->width = params->font_size * params->font_scale * params->table_columns * 4.6;
@@ -859,11 +971,18 @@ parse_overlay_config(struct overlay_params *params,
    auto real_size = params->font_size * params->font_scale;
    real_font_size = ImVec2(real_size, real_size / 2);
    HUDElements.params = params;
-   if (params->enabled[OVERLAY_PARAM_ENABLED_legacy_layout]){
-        HUDElements.legacy_elements();
+
+   for (const auto& option : HUDElements.options) {
+      SPDLOG_DEBUG("Param: '{}' = '{}'", option.first, option.second);
+   }
+
+   if (params->enabled[OVERLAY_PARAM_ENABLED_legacy_layout]) {
+      HUDElements.legacy_elements();
    } else {
-      for (auto& option : HUDElements.options)
+      HUDElements.ordered_functions.clear();
+      for (auto& option : HUDElements.options) {
          HUDElements.sort_elements(option);
+      }
    }
 
    // Needs ImGui context but it is null here for OpenGL so just note it and update somewhere else
@@ -878,8 +997,6 @@ parse_overlay_config(struct overlay_params *params,
       logger->stop_logging();
    }
    logger = std::make_unique<Logger>(params);
-   if(params->autostart_log && !logger->is_active())
-      std::thread(autostart_log, params->autostart_log).detach();
 #ifdef MANGOAPP
    {
       extern bool new_frame;
@@ -890,35 +1007,57 @@ parse_overlay_config(struct overlay_params *params,
    mangoapp_cv.notify_one();
    g_fsrSharpness = params->fsr_steam_sharpness;
 #endif
+   if (HUDElements.net)
+      HUDElements.net->should_reset = true;
+
+   {
+      std::lock_guard<std::mutex> lock(config_mtx);
+      config_ready = true;
+      config_cv.notify_one();
+   }
 }
 
 bool parse_preset_config(int preset, struct overlay_params *params){
-   const std::string data_dir = get_data_dir();
+   const char *presets_file_env = getenv("MANGOHUD_PRESETSFILE");
    const std::string config_dir = get_config_dir();
-   std::string preset_path = config_dir + "/MangoHud/" + "presets.conf";
-   FILE *preset_file = fopen(preset_path.c_str(), "r");
-   char line[20];
-   char preset_string[20];
-   sprintf(preset_string, "[preset %d]\n", preset);
-   bool found_preset = false;
-   if (preset_file == NULL)
-      return false;
+   std::string preset_path = presets_file_env ? presets_file_env : config_dir + "/MangoHud/" + "presets.conf";
 
-   while (fgets(line, sizeof(line), preset_file)){
-      if (strcmp(line, preset_string) == 0){
+   char preset_string[20];
+   snprintf(preset_string, sizeof(preset_string), "[preset %d]", preset);
+
+   std::ifstream stream(preset_path);
+   stream.imbue(std::locale::classic());
+
+   if (!stream.good()) {
+      SPDLOG_DEBUG("Failed to read presets file: '{}'.  Falling back to default presets", preset_path);
+      return false;
+   }
+
+   std::string line;
+   bool found_preset = false;
+
+   while (std::getline(stream, line)) {
+      trim(line);
+
+      if (line == "")
+         continue;
+
+      if (line == preset_string) {
          found_preset = true;
          continue;
       }
 
-      if (found_preset){
-         if(strcmp(line, "preset") == 0 || line[0] == '\n' || line[0] == '\0')
+      if (found_preset) {
+         if (line.front() == '[' && line.back() == ']')
             break;
 
+         if (line == "inherit")
+            presets(preset, params, true);
+
          parseConfigLine(line, params->options);
       }
    }
 
-   fclose(preset_file);
    return found_preset;
 }
 
@@ -927,9 +1066,10 @@ void add_to_options(struct overlay_params *params, std::string option, std::stri
    params->options[option] = value;
 }
 
-void presets(int preset, struct overlay_params *params) {
-   if (parse_preset_config(preset, params))
-      return;
+int i = 0;
+void presets(int preset, struct overlay_params *params, bool inherit) {
+   if (!inherit && parse_preset_config(preset, params))
+         return;
 
    switch(preset) {
       case 0:
@@ -944,8 +1084,9 @@ void presets(int preset, struct overlay_params *params) {
          add_to_options(params, "fps", "1");
          add_to_options(params, "fps_only", "1");
          add_to_options(params, "frametime", "0");
+         add_to_options(params, "debug", "0");
          break;
-      
+
       case 2:
          params->table_columns = 20;
          add_to_options(params, "horizontal", "1");
@@ -964,8 +1105,9 @@ void presets(int preset, struct overlay_params *params) {
          add_to_options(params, "cpu_power", "1");
          add_to_options(params, "battery_watt", "1");
          add_to_options(params, "battery_time", "1");
+         add_to_options(params, "debug", "0");
          break;
-      
+
       case 3:
          add_to_options(params, "cpu_temp", "1");
          add_to_options(params, "gpu_temp", "1");
@@ -977,10 +1119,14 @@ void presets(int preset, struct overlay_params *params) {
          add_to_options(params, "gpu_mem_clock", "1");
          add_to_options(params, "gpu_core_clock", "1");
          add_to_options(params, "battery", "1");
+         add_to_options(params, "hdr", "1");
+         add_to_options(params, "debug", "0");
          break;
 
       case 4:
          add_to_options(params, "full", "1");
+         add_to_options(params, "throttling_status", "0");
+         add_to_options(params, "throttling_status_graph", "0");
          add_to_options(params, "io_read", "0");
          add_to_options(params, "io_write", "0");
          add_to_options(params, "arch", "0");
@@ -991,6 +1137,21 @@ void presets(int preset, struct overlay_params *params) {
          add_to_options(params, "frame_count", "0");
          add_to_options(params, "show_fps_limit", "0");
          add_to_options(params, "resolution", "0");
+         add_to_options(params, "gpu_load_change", "0");
+         add_to_options(params, "core_load_change", "0");
+         add_to_options(params, "cpu_load_change", "0");
+         add_to_options(params, "fps_color_change", "0");
+         add_to_options(params, "hdr", "1");
+         add_to_options(params, "refresh_rate", "1");
+         add_to_options(params, "media_player", "0");
+         add_to_options(params, "debug", "1");
+         add_to_options(params, "version", "0");
+         add_to_options(params, "frame_timing_detailed", "1");
+         add_to_options(params, "network", "1");
+         add_to_options(params, "present_mode", "0");
+         if ( deviceID == 0x1435 || deviceID == 0x163f )
+            add_to_options(params, "gpu_fan", "0");
+
          break;
 
    }
