@@ -13,7 +13,7 @@ void init_d3d_shared(){
     vendorID = get_device_id_dxgi();
     if (cfg_inited)
         return;
-     parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"));
+     parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"), false);
      _params = &params;
      cfg_inited = true;
     //  init_cpu_stats(params);
@@ -22,4 +22,4 @@ void init_d3d_shared(){
 void d3d_run(){
     check_keybinds(params, vendorID);
 	update_hud_info(sw_stats, params, vendorID);
-}
\ No newline at end of file
+}
